import numpy as np
import pandas as pd
from scipy.stats import iqr
from scipy.signal import find_peaks
from datetime import timedelta

from py_agata.input_validator import *
from py_agata.time_in_ranges import time_in_target, time_in_hypoglycemia


def mean_glucose(data):
    """
    Computes the mean glucose level (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    mean_glucose: float
        The mean glucose level.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on mean: https://en.wikipedia.org/wiki/Mean (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Return the result
    return np.mean(values)


def median_glucose(data):
    """
    Computes the median glucose level (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    mean_glucose: float
        The median glucose level.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on median: https://en.wikipedia.org/wiki/Median (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Return the result
    return np.median(values)


def std_glucose(data):
    """
    Computes the std glucose level (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    std_glucose: float
        The std glucose level.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on standard deviation: https://en.wikipedia.org/wiki/Standard_deviation (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Return the result
    return np.std(values, ddof=1)


def cv_glucose(data):
    """
    Computes the coefficient of variation of glucose (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    cv_glucose: float
        The cv of glucose.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on coefficient of variation: https://en.wikipedia.org/wiki/Coefficient_of_variation (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Return the result
    return 100 * std_glucose(data) / mean_glucose(data)


def range_glucose(data):
    """
    Computes the spanned glucose range (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    range: float
        The range of glucose.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on range: https://en.wikipedia.org/wiki/Range_(statistics) (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Return the result
    if(data.glucose.values.size == 0):
        return np.nan
    return np.nanmax(data.glucose.values) - np.nanmin(data.glucose.values)


def iqr_glucose(data):
    """
    Computes the interquartile range of glucose values (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    iqr_glucose: float
        The interquartile range of glucose.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Wikipedia on IQR: https://en.wikipedia.org/wiki/Interquartile_range (Accessed: 2020-12-10).
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get rid of nans
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return the result
    return iqr(values)


def auc_glucose_over_basal(data, basal):
    """
    Computes the area under the glucose curve using a given basal offset (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    auc_glucose_over_basal: float
        The area under the glucose curve.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_float_parameter(basal)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Shift the trace
    values = values - basal

    # Get ts
    t0 = pd.to_datetime(data.t.values[0]).to_pydatetime()
    t1 = pd.to_datetime(data.t.values[1]).to_pydatetime()
    ts = (t1 - t0).total_seconds() / 60

    # Return the result
    return np.sum(values*ts)


def auc_glucose(data):
    """
    Computes the area under the glucose curve (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    auc_glucose: float
        The area under the glucose curve.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Return results
    return auc_glucose_over_basal(data, 0.)


def gmi(data):
    """
    Computes the glucose management indicator of the given data (ignoring nan values).
    It should be computed only if more than 12 days are available.

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    gmi: float
        The glucose management indicator of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Bergenstal et al., "Glucose Management Indicator (GMI): A new term
    for estimating A1C from continuous glucose monitoring", Diabetes Care,
    2018, vol. 41, pp. 2275-2280. DOI: 10.2337/dc18-1581.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Return results
    return 3.31 + 0.02392 * mean_glucose(data)


def cogi(data):
    """
    Computes the Continuous Glucose Monitoring Index (COGI) of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    cogi: float
        The Continuous Glucose Monitoring Index (COGI) of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Leelaranthna et al., "Evaluating glucose control with a novel composite
    Continuous Glucose Monitoring Index", Journal of Diabetes Science and Technology,
    2019, vol. 14, pp. 277-283. DOI: 10.1177/1932296819838525.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Compute TIR component
    tir = time_in_target(data)*0.5

    # Compute TBR component
    tbr = np.min([15, time_in_hypoglycemia(data)])
    tbr = (100 - 100 / 15 * tbr) * 0.35

    # Compute GV component
    gv = np.min([np.max([std_glucose(data) / 18.018, 1]), 6])
    gv = (120 - 20 * gv) * 0.15

    # Return results
    return tir + tbr + gv


def conga(data):
    """
    Computes the Continuous Overall Net Glycemic Action (CONGA) of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    conga: float
        The Continuous Overall Net Glycemic Action (CONGA) of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    McDonnell et al., "A novel approach to continuous glucose analysis
    utilizing glycemic variation", Diabetes Technol Ther, 2005, vol. 7,
    pp. 253–263. DOI: 10.1089/dia.2005.7.253.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Set the CONGAOrd hyperparameter to 4 (number of hours in the past it
    # refers to)
    conga_ord = 4

    # Build vectors
    n = data.glucose.values.size
    dc = np.empty(shape=(0,))

    for i in range(1,n):

        # Find the index referring to conga_ord hours ago
        j = np.where(data.t <= (pd.to_datetime(data.t.values[i]).to_pydatetime() - timedelta(hours=conga_ord)))[0]

        if not j.size == 0:
            j = j[-1]
            dc = np.append(dc, data.glucose.values[i] - data.glucose[j])

    # Return results
    if dc.size == 0:
        return np.nan
    else:
        return np.nanstd(dc, ddof=1)


def j_index(data):
    """
    Computes the J-Index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    j_index: float
        The J-Index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Wójcicki, "J-index. A new proposition of the assessment of current
    glucose control in diabetic patients", Hormone and Metabolic Reseach,
    1995, vol. 27, pp. 41-42. DOI: 10.1055/s-2007-979906.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    return 1e-3 * (mean_glucose(data) + std_glucose(data)) ** 2


def mage_plus_index(data):
    """
    Computes the mean amplitude of positive glycemic excursion (MAGE+) index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    mage_plus_index: float
        The mean amplitude of positive glycemic excursion (MAGE+) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Service et al., "Mean amplitude of glycemic excursions, a measure of
    diabetic instability", Diabetes, 1970, vol. 19, pp. 644-655. DOI:
    10.2337/diab.19.9.644.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get the first and last day limits
    if data.t.values.size == 0:
        return np.nan

    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day + 1, hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate
    n_days = (last_day - first_day).days
    mage_day_plus = np.empty(shape=(n_days,))

    for d in range(0, n_days):

        # Step 0: parameters

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d + 1))
        flags = np.logical_and(low_limit, high_limit)
        day_data = data.glucose.values[flags]

        # Get glucose values (might be nan)
        std_within = np.nanstd(day_data, ddof=1)
        n = day_data.size

        if n > 3:
            # Step 1: turning points are only local extrema
            i_max = find_peaks(day_data)[0]
            i_min = find_peaks(-day_data)[0]
            i_turning = np.union1d([0, n-1], np.union1d(i_max, i_min)).astype(int)

            turning = day_data[i_turning]
            n_turning = i_turning.size

            # Step 2: Turning points of no interest are removed
            # A turning point is removed if it's not significantly different from
            # BOTH its left and right-hand side RETAINED neighbours.

            to_be_kept = [True] * n_turning

            for i in range(1, n_turning-1): # First and last samples are retained

                condition_1 = abs(turning[i] - turning[i - 1]) < std_within
                condition_2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition_1 and condition_2)

            i_turning = i_turning[to_be_kept]

            # Step 3: Turning points are removed again or moved appropriately
            i = 1
            while i < len(i_turning)-1:
                prev = i_turning[i - 1]
                curr = i_turning[i]
                next = i_turning[i + 1]
                prev_slope = day_data[curr] - day_data[prev]
                next_slope = day_data[next] - day_data[curr]

                if prev_slope < 0 and next_slope > 0:  # Minimum
                    # The actual current turning point is the min in the interval
                    temp = np.nanargmin(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the max to the left of the current turning point.
                    temp = np.nanargmax(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the max to the right of the current turning point.
                    temp = np.nanargmax(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                elif prev_slope > 0 and next_slope < 0:  # Maximum
                    # The actual current turning point is the max in the interval
                    temp = np.nanargmax(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the min to the left of the current turning point.
                    temp = np.nanargmin(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the min to the right of the current turning point.
                    temp = np.nanargmin(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                else:  # Middle point
                    i_turning = np.delete(i_turning, i)

            # Step 4: Remove residual spurious turning points.
            # Turning points not significantly different from EITHER neighbour are
            # removed. Some extra processing is needed for the first and last sample.
            sample1 = day_data[i_turning[0]]
            sample2 = day_data[i_turning[1]]

            if abs(sample2 - sample1) < std_within:
                i_turning = np.delete(i_turning, 0)

            if len(i_turning) > 1:
                # Last sample processing
                sample1 = day_data[i_turning[-2]]
                sample2 = day_data[i_turning[-1]]
                if abs(sample2 - sample1) < std_within:
                    i_turning = np.delete(i_turning, len(i_turning)-1)

            turning = day_data[i_turning]
            n_turning = len(i_turning)

            # Internal points
            to_be_kept = np.ones(n_turning, dtype=bool)
            for i in range(1, n_turning - 1):
                condition1 = abs(turning[i] - turning[i - 1]) < std_within
                condition2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition1 or condition2)

            i_turning = i_turning[to_be_kept]
            turning = day_data[i_turning]

            # Step 5: Compute daily MAGE+
            excursions = np.diff(turning)
            mage_day_plus[d] = np.nanmean(excursions[excursions > 0])
        else:
            mage_day_plus[d] = np.nan

    # Compute index
    mage_day_plus[np.isnan(mage_day_plus)] = 0  # Correct for 'mean' behavior
    mpi = np.mean(mage_day_plus)

    # Manage all nan data
    if np.all(np.isnan(data['glucose'])):
        mpi = np.nan

    return mpi


def mage_minus_index(data):
    """
    Computes the mean amplitude of negative glycemic excursion (MAGE-) index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    mage_minus_index: float
        The mean amplitude of negative glycemic excursion (MAGE+) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Service et al., "Mean amplitude of glycemic excursions, a measure of
    diabetic instability", Diabetes, 1970, vol. 19, pp. 644-655. DOI:
    10.2337/diab.19.9.644.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan
    # Get the first and last day limits
    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day + 1, hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate
    n_days = (last_day - first_day).days
    mage_day_minus = np.empty(shape=(n_days,))

    for d in range(0, n_days):

        # Step 0: parameters

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d + 1))
        flags = np.logical_and(low_limit, high_limit)
        day_data = data.glucose.values[flags]

        # Get glucose values (might be nan)
        std_within = np.nanstd(day_data, ddof=1)
        n = day_data.size

        if n > 3:
            # Step 1: turning points are only local extrema
            i_max = find_peaks(day_data)[0]
            i_min = find_peaks(-day_data)[0]
            i_turning = np.union1d([0, n-1], np.union1d(i_max, i_min)).astype(int)

            turning = day_data[i_turning]
            n_turning = i_turning.size

            # Step 2: Turning points of no interest are removed
            # A turning point is removed if it's not significantly different from
            # BOTH its left and right-hand side RETAINED neighbours.

            to_be_kept = [True] * n_turning

            for i in range(1, n_turning-1): # First and last samples are retained

                condition_1 = abs(turning[i] - turning[i - 1]) < std_within
                condition_2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition_1 and condition_2)

            i_turning = i_turning[to_be_kept]

            # Step 3: Turning points are removed again or moved appropriately
            i = 1
            while i < len(i_turning)-1:
                prev = i_turning[i - 1]
                curr = i_turning[i]
                next = i_turning[i + 1]
                prev_slope = day_data[curr] - day_data[prev]
                next_slope = day_data[next] - day_data[curr]

                if prev_slope < 0 and next_slope > 0:  # Minimum
                    # The actual current turning point is the min in the interval
                    temp = np.nanargmin(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the max to the left of the current turning point.
                    temp = np.nanargmax(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the max to the right of the current turning point.
                    temp = np.nanargmax(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                elif prev_slope > 0 and next_slope < 0:  # Maximum
                    # The actual current turning point is the max in the interval
                    temp = np.nanargmax(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the min to the left of the current turning point.
                    temp = np.nanargmin(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the min to the right of the current turning point.
                    temp = np.nanargmin(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                else:  # Middle point
                    i_turning = np.delete(i_turning, i)

            # Step 4: Remove residual spurious turning points.
            # Turning points not significantly different from EITHER neighbour are
            # removed. Some extra processing is needed for the first and last sample.
            sample1 = day_data[i_turning[0]]
            sample2 = day_data[i_turning[1]]

            if abs(sample2 - sample1) < std_within:
                i_turning = np.delete(i_turning, 0)

            if len(i_turning) > 1:
                # Last sample processing
                sample1 = day_data[i_turning[-2]]
                sample2 = day_data[i_turning[-1]]
                if abs(sample2 - sample1) < std_within:
                    i_turning = np.delete(i_turning, len(i_turning)-1)

            turning = day_data[i_turning]
            n_turning = len(i_turning)

            # Internal points
            to_be_kept = np.ones(n_turning, dtype=bool)
            for i in range(1, n_turning - 1):
                condition1 = abs(turning[i] - turning[i - 1]) < std_within
                condition2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition1 or condition2)

            i_turning = i_turning[to_be_kept]
            turning = day_data[i_turning]

            # Step 5: Compute daily MAGE+
            excursions = np.diff(turning)
            mage_day_minus[d] = np.nanmean(excursions[excursions < 0])
        else:
            mage_day_minus[d] = np.nan

    # Compute index
    mage_day_minus[np.isnan(mage_day_minus)] = 0  # Correct for 'mean' behavior
    mmi = -np.mean(mage_day_minus)

    # Manage all nan data
    if np.all(np.isnan(data['glucose'])):
        mmi = np.nan

    return mmi


def mage_index(data):
    """
    Computes the mean amplitude of glycemic excursion (MAGE) index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    mage_index: float
        The mean amplitude of glycemic excursion (MAGE) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Service et al., "Mean amplitude of glycemic excursions, a measure of
    diabetic instability", Diabetes, 1970, vol. 19, pp. 644-655. DOI:
    10.2337/diab.19.9.644.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if np.isnan(mage_plus_index(data)) and np.isnan(mage_minus_index(data)):
        return np.nan
    return np.nanmean([mage_plus_index(data), mage_minus_index(data)])


def ef_index(data):
    """
    Computes the excursion frequency (EF) index of the given data, i.e., the number of excursion > 75 (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    ef_index: float
        The excursion frequency (EF) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Service et al., "Mean amplitude of glycemic excursions, a measure of
    diabetic instability", Diabetes, 1970, vol. 19, pp. 644-655. DOI:
    10.2337/diab.19.9.644.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan

    # Set the fixed parameter
    ef_th = 75

    # Get the first and last day limits
    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day + 1, hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate
    n_days = (last_day - first_day).days
    ef_day = np.empty(shape=(n_days,))

    for d in range(0, n_days):

        # Step 0: parameters

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d + 1))
        flags = np.logical_and(low_limit, high_limit)
        day_data = data.glucose.values[flags]

        # Get glucose values (might be nan)
        std_within = np.nanstd(day_data, ddof=1)
        n = day_data.size

        if n > 3:
            # Step 1: turning points are only local extrema
            i_max = find_peaks(day_data)[0]
            i_min = find_peaks(-day_data)[0]
            i_turning = np.union1d([0, n-1], np.union1d(i_max, i_min)).astype(int)

            turning = day_data[i_turning]
            n_turning = i_turning.size

            # Step 2: Turning points of no interest are removed
            # A turning point is removed if it's not significantly different from
            # BOTH its left and right-hand side RETAINED neighbours.

            to_be_kept = [True] * n_turning

            for i in range(1, n_turning-1): # First and last samples are retained

                condition_1 = abs(turning[i] - turning[i - 1]) < std_within
                condition_2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition_1 and condition_2)

            i_turning = i_turning[to_be_kept]

            # Step 3: Turning points are removed again or moved appropriately
            i = 1
            while i < len(i_turning)-1:
                prev = i_turning[i - 1]
                curr = i_turning[i]
                next = i_turning[i + 1]
                prev_slope = day_data[curr] - day_data[prev]
                next_slope = day_data[next] - day_data[curr]

                if prev_slope < 0 and next_slope > 0:  # Minimum
                    # The actual current turning point is the min in the interval
                    temp = np.nanargmin(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the max to the left of the current turning point.
                    temp = np.nanargmax(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the max to the right of the current turning point.
                    temp = np.nanargmax(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                elif prev_slope > 0 and next_slope < 0:  # Maximum
                    # The actual current turning point is the max in the interval
                    temp = np.nanargmax(day_data[prev:(next+1)]) + prev
                    curr = temp
                    i_turning[i] = curr
                    # The actual previous turning point is the min to the left of the current turning point.
                    temp = np.nanargmin(day_data[prev:curr]) + prev
                    i_turning[i - 1] = temp
                    # The actual following turning point is the min to the right of the current turning point.
                    temp = np.nanargmin(day_data[(curr + 1):(next+1)]) + curr + 1
                    i_turning[i + 1] = temp

                    i += 1
                else:  # Middle point
                    i_turning = np.delete(i_turning, i)

            # Step 4: Remove residual spurious turning points.
            # Turning points not significantly different from EITHER neighbour are
            # removed. Some extra processing is needed for the first and last sample.
            sample1 = day_data[i_turning[0]]
            sample2 = day_data[i_turning[1]]

            if abs(sample2 - sample1) < std_within:
                i_turning = np.delete(i_turning, 0)

            if len(i_turning) > 1:
                # Last sample processing
                sample1 = day_data[i_turning[-2]]
                sample2 = day_data[i_turning[-1]]
                if abs(sample2 - sample1) < std_within:
                    i_turning = np.delete(i_turning, len(i_turning)-1)

            turning = day_data[i_turning]
            n_turning = len(i_turning)

            # Internal points
            to_be_kept = np.ones(n_turning, dtype=bool)
            for i in range(1, n_turning - 1):
                condition1 = abs(turning[i] - turning[i - 1]) < std_within
                condition2 = abs(turning[i + 1] - turning[i]) < std_within
                to_be_kept[i] = not(condition1 or condition2)

            i_turning = i_turning[to_be_kept]
            turning = day_data[i_turning]

            # Step 5: Compute daily MAGE+
            excursions = np.diff(turning)
            ef_day[d] = np.where(abs(excursions) > ef_th)[0].size
        else:
            ef_day[d] = np.nan

    # Compute index
    ei = np.nansum(ef_day)/n_days

    # Manage all nan data
    if np.all(np.isnan(data['glucose'])):
        ei = np.nan

    return ei


def modd(data):
    """
    Computes the mean of daily differences (MODD) of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    modd: float
        The mean of daily differences (MODD) of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Molnar et al., "Day-to-day variation of continuously monitored
    glycaemia: a further measure of diabetic instability", Diabetologia,
    1972, vol. 8, pp. 342–348. DOI: 10.1007/BF01218495.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Build vectors
    yesterday = timedelta(minutes=1440)

    n = len(data)

    Dm = []


    for i in range(1, n):

        # Find the index referring to the same time yesterday
        j = np.where(data.t <= (pd.to_datetime(data.t.values[i]).to_pydatetime() - yesterday))[0]

        if j.size > 0:  # if there is a meaningful sample in data[j]
            j = j[-1]
            Dm.append(abs(data.glucose.values[i] - data.glucose.values[j]))

    if Dm:
        modd = np.nanmean(Dm)
    else:
        modd = np.nan
    return modd


def sddm_index(data):
    """
    Computes the standard deviation of within-day means (SDDM) index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    sddm_index: float
        The standard deviation of within-day means (SDDM) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Rodbard et al., "New and improved methods to characterize glycemic
    variability using continuous glucose monitoring", Diabetes Technology &
    Therapeutics, 2009, vol. 11, pp. 551-565. DOI: 10.1089/dia.2009.0015.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan
    # Get the first and last day limits
    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day + 1, hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate
    n_days = (last_day - first_day).days
    mean_within = np.zeros(shape=(n_days,))

    for d in range(0, n_days):

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d + 1))
        flags = np.logical_and(low_limit, high_limit)
        day_data = data.glucose.values[flags]

        # Get daily mean and std
        mean_within[d] = np.nanmean(day_data)

    # Compute index
    return np.nanstd(mean_within, ddof=1)


def sdw_index(data):
    """
    Computes the mean of within-day standard deviation (SDW) index of the given data (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    sdw_index: float
        The mean of within-day standard deviation (SDW) index of the given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Rodbard et al., "New and improved methods to characterize glycemic
    variability using continuous glucose monitoring", Diabetes Technology &
    Therapeutics, 2009, vol. 11, pp. 551-565. DOI: 10.1089/dia.2009.0015.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan
    # Get the first and last day limits
    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day + 1, hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate
    n_days = (last_day - first_day).days
    std_within = np.zeros(shape=(n_days,))

    for d in range(0, n_days):

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d + 1))
        flags = np.logical_and(low_limit, high_limit)
        day_data = data.glucose.values[flags]

        # Get daily mean and std
        std_within[d] = np.nanstd(day_data, ddof=1)

    # Compute index
    return np.nanmean(std_within)

def glucose_roc(data):
    """
    Computes the glucose rate-of-change (ROC) trace.
    As defined in the given reference, ROC at time t is defined as the difference
    between the glucose at time t and t-15 minutes divided by 15. By
    definition, the first two samples are always nan (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    glucose_roc: pd.Dataframe
        Dataframe with column `t` and `glucose_roc` containing the glucose ROC (in mg/dl/min).

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Clarke et al., "Statistical Tools to Analyze Continuous Glucose
    Monitor Data", Diabetes Technol Ther, 2009,
    vol. 11, pp. S45-S54. DOI: 10.1089=dia.2008.0138.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    g_roc = np.empty(shape=(data.glucose.values.size,))
    g_roc.fill(np.nan)

    if g_roc.size > 4:

        for t in range(3, g_roc.size):

            g_roc[t] = (data.glucose.values[t] - data.glucose.values[t-3]) / 15

    return pd.DataFrame(data={'t': data.t.values, 'glucose_roc': g_roc})


def std_glucose_roc(data):
    """
    Computes the standard of glucose rate-of-change of given data (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    std_glucose_roc: float
        The standard of glucose rate-of-change of given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Clarke et al., "Statistical Tools to Analyze Continuous Glucose
    Monitor Data", Diabetes Technol Ther, 2009,
    vol. 11, pp. S45-S54. DOI: 10.1089=dia.2008.0138.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    roc = glucose_roc(data)

    return np.nanstd(roc.glucose_roc.values, ddof=1)


def cvga(data):
    """
    Computes the standard of glucose rate-of-change of given data (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    std_glucose_roc: float
        The standard of glucose rate-of-change of given data.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    - Magni et al., "Evaluating the efficacy of closed-loop glucose
    regulation via control-variability grid analysis", Journal of Diabetes
    Science and Technology, 2008, vol. 2, pp. 630-635. DOI:
    10.1177/193229680800200414.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan

    roc = glucose_roc(data)

    x = np.min([np.max([110 - np.nanmin(data.glucose.values), 0]), 60])
    p = np.polyfit([110, 180, 300, 400], [0, 20, 40, 60], 3)
    y = np.polyval(p, np.nanmax(data.glucose.values))

    return x**2 + y**2



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.glycemic_transformation import grade_eu_score


def test_grade_eu_score():
    """
    Unit test of grade_eu_score function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    #Tests
    assert np.isnan(grade_eu_score(data)) == False
    assert np.round(grade_eu_score(data)*1000)/1000 == 3.608

    # Set empty data
    t = np.arange(datetime(2000, 1, 1, 1, 0, 0), datetime(2000, 1, 1, 1, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = np.nan
    glucose[1:3] = [np.nan, np.nan]
    glucose[3] = np.nan
    glucose[4:6] = [np.nan, np.nan]
    glucose[6:8] = [np.nan, np.nan]
    glucose[8:10] = [np.nan, np.nan]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # Tests
    assert np.isnan(grade_eu_score(data))



import numpy as np
import pandas as pd

from datetime import datetime, timedelta
from copy import copy

from py_agata.input_validator import *

def rmse(data, data_hat):
    """
    Computes the root mean squared error (RMSE) between two glucose traces (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data

    Returns
    -------
    rmse: float
        The computed root mean squared error (mg/dl).

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Wikipedia on RMSE: https://en.wikipedia.org/wiki/Root-mean-square_deviation. (Accessed: 2020-12-10)
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)

    if data.glucose.values.size == 0:
        return np.nan
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return np.nan
    return np.sqrt(np.mean((data.glucose.values[idxs] - data_hat.glucose.values[idxs]) ** 2))


def mard(data, data_hat):
    """
    Computes the mean absolute relative difference (MARD) between two glucose traces (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data

    Returns
    -------
    mard: float
        The computed MARD (%).

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Gini, "Measurement of Inequality and Incomes", The Economic Journal,
    vol. 31, 1921, pp. 124–126. DOI:10.2307/2223319.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)

    if data.glucose.values.size == 0:
        return np.nan
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return np.nan
    return 100 * np.mean(np.abs(np.divide(data.glucose.values[idxs] - data_hat.glucose.values[idxs],data.glucose.values[idxs])))


def cod(data, data_hat):
    """
    Computes the coefficient of determination (COD) between two glucose traces (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data

    Returns
    -------
    cod: float
        The computed COD (%).

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Wright, "Correlation and causation", Journal of Agricultural Research, vol. 20, 1921, pp. 557–585.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)

    if data.glucose.values.size == 0:
        return np.nan
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return np.nan
    residuals = data.glucose.values[idxs] - data_hat.glucose.values[idxs]
    return 100 * (1 - np.linalg.norm(residuals,ord=2)**2 / np.linalg.norm(data.glucose.values[idxs] - np.mean(data.glucose.values[idxs]),ord=2)**2)

def clarke(data, data_hat):
    """
    Computes the Clarke Error Grid Analysis (CEGA) (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data

    Returns
    -------
    clarke: dict
        A dictionary containing the results of the CEGA with fields:
        - a: float
            The percentage of time spent in Zone A
        - b: float
            The percentage of time spent in Zone B
        - c: float
            The percentage of time spent in Zone C
        - d: float
            The percentage of time spent in Zone D
        - e: float
            The percentage of time spent in Zone E

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Clarke et al., "Evaluating clinical accuracy of systems for self-monitoring
    of blood glucose", Diabetes Care, 1987, vol. 10, pp. 622–628. DOI: 10.2337/diacare.10.5.622.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)

    results = dict()
    results["a"] = np.nan
    results["b"] = np.nan
    results["c"] = np.nan
    results["d"] = np.nan
    results["e"] = np.nan
    if data.glucose.values.size == 0:
        return results
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return results

    y = data.glucose.values[idxs]
    yp = data_hat.glucose.values[idxs]

    total = np.zeros(shape=(5,))
    n = y.size

    for i in range(n):

        if (yp[i] <= 70 and y[i] <= 70) or (yp[i] <= 1.2*y[i] and yp[i] >= 0.8*y[i]):
            total[0] += 1
        elif (y[i] >= 180 and yp[i] <= 70) or (y[i] <= 70 and yp[i] >= 180):
            total[4] += 1
        elif ((y[i] >= 70 and y[i] <= 290) and (yp[i] >= y[i] + 110)) or ( (y[i] >= 130 and y[i] <= 180) and (yp[i] <= 7/5*y[i] - 182)):
            total[2] += 1
        elif (y[i] >= 240 and (yp[i] >= 70 and yp[i] <= 180)) or (y[i] <= 175/3 and yp[i] <= 180 and yp[i] >= 70) or ((y[i] >= 175/3 and y[i] <= 70) and yp[i] >= 6/5*y[i]):
            total[3] += 1
        else:
            total[1] += 1

    total = total/n*100
    results["a"] = total[0]
    results["b"] = total[1]
    results["c"] = total[2]
    results["d"] = total[3]
    results["e"] = total[4]

    return results

def c2_sigmoid(x_vector, a, d, type):

    if a.size == 1:
        a = a*np.ones(shape=(x_vector.size,))

    # Auxiliary function
    y = np.zeros(shape=(x_vector.size,))

    for i in range(x_vector.size):

        x = x_vector[i]

        if type == '>=':
            x_i = 2 / d * (x - a[i] - d/2)
        elif type == '<=':
            x_i = 2 / d * (x - a[i] + d / 2)

        if x_i <= -1:
            y[i] = 0
        elif x_i >= 1:
            y[i] = 1
        elif x_i <= 0:
            y[i] = 0.5 * (-(x_i**4) - 2*(x_i**3) + 2*x_i + 1)
        else:
            y[i] = 0.5 * (x_i ** 4 - 2 * (x_i ** 3) + 2 * x_i + 1)

    return y

def g_rmse(data, data_hat):
    """
    Computes the glucose root mean squared erro (gRMSE) (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data

    Returns
    -------
    g_rmse: float
        The computed glucose root mean squared error (mg/dl).

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Del Favero et al., "A glucose-specific metric to assess predictors and
    identify models", IEEE Transactions on Biomedical Engineering, 2012,
    vol. 59, pp. 1281-1290. DOI: 10.1109/TBME.2012.2185234.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)

    if data.glucose.values.size == 0:
        return np.nan
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return np.nan

    y = data.glucose.values[idxs]
    yp = data_hat.glucose.values[idxs]

    # Parameters
    alpha_l = 1.5
    alpha_h = 1
    d_l1 = 10
    d_l2 = 30
    d_h1 = 20
    d_h2 = 100

    # Compute the cost
    term_l = alpha_l * c2_sigmoid(y, yp, d_l1,'<=') * c2_sigmoid(y, np.array([80]),d_l2,'<=')
    term_h = alpha_h * c2_sigmoid(y, yp, d_h1, '>=') * c2_sigmoid(y, np.array([250]), d_h2, '>=')
    error_grid_inspired_cost = 1 + term_l + term_h

    # Compute the quadratic cost function
    quadratic_cost = (y - yp)*(y - yp)

    # Compute the gMSE
    g_mse = np.nanmean(quadratic_cost * error_grid_inspired_cost)

    # Compute the gRMSE
    return np.sqrt(g_mse)


def time_delay(data, data_hat, ph):
    """
    Computes the delay of a predicted glucose trace.
    The time delay is computed as the time shift necessary to maximize the
    correlation between the two traces.

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe, numpy array or float containing the glucose data
    data_hat: pd.DataFrame
        Pandas dataframe, numpy array or float containing the inferred glucose data
    ph: int

    Returns
    -------
    time_delay: int
        The computed delay (min)

    Raises
    ------
    None

    See Also
    --------
    None
Ï
    Examples
    --------
    None

    References
    ----------
    Del Favero et al., "A glucose-specific metric to assess predictors and
    identify models", IEEE Transactions on Biomedical Engineering, 2012,
    vol. 59, pp. 1281-1290. DOI: 10.1109/TBME.2012.2185234.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_dataframe(data_hat)
    check_data_columns(data_hat)
    check_homogeneous_timegrid(data_hat)
    check_comparable_data(data, data_hat)
    check_int_parameter(ph)

    if data.glucose.values.size == 0:
        return np.nan
    idxs = np.where(np.logical_and(~np.isnan(data.glucose.values), ~np.isnan(data_hat.glucose.values)))[0]
    if idxs.size == 0:
        return np.nan

    y = data.glucose.values[idxs]
    yp = data_hat.glucose.values[idxs]

    t0 = pd.to_datetime(data.t.values[0]).to_pydatetime()
    t1 = pd.to_datetime(data.t.values[1]).to_pydatetime()
    sample_time = int((t1 - t0).total_seconds() / 60)

    errors = np.zeros(shape=(int(ph/sample_time)+1,))

    for steps in range(errors.size):
            errors[steps] = np.sqrt(np.mean((y - yp) ** 2))
            yp = yp[1:]
            y = y[0:-1]

    idx_min_error = np.where(np.min(errors)==errors)[0][0]
    return ph - sample_time*idx_min_error



import numpy as np
import pandas as pd
from scipy.stats import iqr
from scipy.signal import find_peaks
from datetime import timedelta

from py_agata.time_in_ranges import time_in_target, time_in_hypoglycemia
from py_agata.input_validator import *

def mr_index(data, r=100):
    """
    Computes the mr value by Schlichtkrull (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)
    r: int, optional, default : 100
        Hyperparameter for mr value calculation

    Returns
    -------
    mr_index: float
        The mr value

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Schlichtkrull et al., "The M-value, an index of blood-sugar control in diabetics", Acta Medica Scandinavica, 1965,
    vol. 177, pp. 95-102. DOI: 10.1111/j.0954-6820.1965.tb01810.x
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_int_parameter(r)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Return the result
    trans_data = 1000 * abs(np.log10(values / r))**3
    return np.mean(trans_data)


def hypo_index(data):
    """
    Computes the hypoglycemic index by Rodbard (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    hypo_index: float
        The hypo index

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Rodbard et al., "Interpretation of continuous glucose monitoring
    data: glycemic variability and quality of glycemic control", Diabetes
    %  Technology & Therapeutics, 2009, vol. 11, pp. S55-S67. DOI: 10.1089/dia.2008.0132.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Set up the formula parameters
    b = 2
    d = 30
    lltr = 70

    # Compute metric
    return np.sum((lltr - values[values < lltr])**b) / (values.size * d)


def hyper_index(data):
    """
    Computes the hyperglycemic index by Rodbard (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    hyper_index: float
        The hyper index

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Rodbard et al., "Interpretation of continuous glucose monitoring
    data: glycemic variability and quality of glycemic control", Diabetes
    %  Technology & Therapeutics, 2009, vol. 11, pp. S55-S67. DOI: 10.1089/dia.2008.0132.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Set up the formula parameters
    a = 1.1
    c = 30
    ultr = 180

    # Compute metric
    return np.sum((values[values > ultr] - ultr)**a) / (values.size * c)


def igc(data):
    """
    Computes the index of glycemic control by Rodbard (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    igc: float
        The index of glycemic control

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Rodbard et al., "Interpretation of continuous glucose monitoring
    data: glycemic variability and quality of glycemic control", Diabetes
    %  Technology & Therapeutics, 2009, vol. 11, pp. S55-S67. DOI: 10.1089/dia.2008.0132.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Compute metric
    return hypo_index(data) + hyper_index(data)


def grade_hypo_score(data):
    """
    Computes the glycemic risk assessment diabetes equation score in the hypoglycemic range (GRADEhypo)
    by Hill (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    grade_hypo_score: float
        The glycemic risk assessment diabetes equation score in the hypoglycemic range (GRADEhypo) (%).

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Hill et al., "A method for assessing quality of control from
    glucose profiles", Diabetic Medicine , 2007, vol. 24, pp. 753-758.
    DOI: 10.1111/j.1464-5491.2007.02119.x.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Set up the formula parameters
    grade = 425 * (np.log10(np.log10(values / 18)) + .16)**2
    g_tot = np.sum(grade)
    return 100 * np.sum(grade[values < 70]) / g_tot


def grade_hyper_score(data):
    """
    Computes the glycemic risk assessment diabetes equation score in the hyperglycemic range (GRADEhyper)
    by Hill (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    grade_hyper_score: float
        The glycemic risk assessment diabetes equation score in the hyperglycemic range (GRADEhyper) (%).

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Hill et al., "A method for assessing quality of control from
    glucose profiles", Diabetic Medicine , 2007, vol. 24, pp. 753-758.
    DOI: 10.1111/j.1464-5491.2007.02119.x.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Set up the formula parameters
    grade = 425 * (np.log10(np.log10(values / 18)) + .16)**2
    g_tot = np.sum(grade)
    return 100 * np.sum(grade[values > 180]) / g_tot


def grade_eu_score(data):
    """
    Computes the glycemic risk assessment diabetes equation score in the euglycemic range (GRADEeu)
    by Hill (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    grade_eu_score: float
        The glycemic risk assessment diabetes equation score in the euglycemic range (GRADEeu) (%).

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Hill et al., "A method for assessing quality of control from
    glucose profiles", Diabetic Medicine , 2007, vol. 24, pp. 753-758.
    DOI: 10.1111/j.1464-5491.2007.02119.x.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    return 100 - (grade_hypo_score(data) + grade_hyper_score(data))


def grade_score(data):
    """
    Computes the glycemic risk assessment diabetes equation score (GRADE)
    by Hill (ignores nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)

    Returns
    -------
    grade_hyper_score: float
        The glycemic risk assessment diabetes equation score (GRADE) (%).

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Hill et al., "A method for assessing quality of control from
    glucose profiles", Diabetic Medicine , 2007, vol. 24, pp. 753-758.
    DOI: 10.1111/j.1464-5491.2007.02119.x.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Get non-nan values
    values = data.glucose.values[~np.isnan(data.glucose.values)]

    # Return nan if all values are nan
    if values.size == 0:
        return np.nan

    # Set up the formula parameters
    grade = 425 * (np.log10(np.log10(values / 18)) + .16)**2
    return np.mean(grade)



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.error import clarke


def test_clarke():
    """
    Unit test of clarke function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [180, 180]
    glucose[6:8] = [170, 170]
    glucose[8:10] = [260, 100]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 80
    glucose[1:3] = [80, 80]
    glucose[3] = 70
    glucose[4:6] = [130, 130]
    glucose[6:8] = [np.nan, np.nan]
    glucose[8:10] = [60, 220]
    glucose[10] = 60
    d = {'t': t, 'glucose': glucose}
    data_hat = pd.DataFrame(data=d)

    #Tests
    results = clarke(data,data_hat)
    assert type(results) is dict
    assert results["a"] == 12.5
    assert results["b"] == 25
    assert results["c"] == 12.5
    assert results["d"] == 37.5
    assert results["e"] == 12.5

    d = {'t': [], 'glucose': []}
    data_hat = pd.DataFrame(data=d)
    d = {'t': [], 'glucose': []}
    data = pd.DataFrame(data=d)

    # Tests
    assert np.isnan(clarke(data, data_hat)["a"])
    assert np.isnan(clarke(data, data_hat)["b"])
    assert np.isnan(clarke(data, data_hat)["c"])
    assert np.isnan(clarke(data, data_hat)["d"])
    assert np.isnan(clarke(data, data_hat)["e"])

    d = {'t': [t[0]], 'glucose': [np.nan]}
    data_hat = pd.DataFrame(data=d)
    d = {'t': [t[0]], 'glucose': [120]}
    data = pd.DataFrame(data=d)

    # Tests
    assert np.isnan(clarke(data, data_hat)["a"])
    assert np.isnan(clarke(data, data_hat)["b"])
    assert np.isnan(clarke(data, data_hat)["c"])
    assert np.isnan(clarke(data, data_hat)["d"])
    assert np.isnan(clarke(data, data_hat)["e"])


import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.inspection import find_nan_islands


def test_find_nan_islands():
    """
    Unit test of find_nan_islands function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 0, 0)+timedelta(minutes=120), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.size,))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:9] = [np.nan, np.nan, np.nan, np.nan, np.nan]
    glucose[10:12] = [200, 200]
    glucose[12:15] = [260, 260, 260]
    glucose[15:17] = [np.nan, np.nan]
    glucose[17] = 180
    glucose[18] = np.nan
    glucose[19] = 140
    glucose[20:24] = [np.nan, np.nan, np.nan, np.nan]
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # 1. check NaN presence
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data,0)
    assert np.any(np.isnan(short_nan)) == False
    assert np.any(np.isnan(long_nan)) == False
    assert np.any(np.isnan(nan_start)) == False
    assert np.any(np.isnan(nan_end)) == False

    # 2. check nan_start calculation
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 2)
    assert nan_start[0] == 4
    assert nan_start[1] == 15
    assert nan_start[2] == 18
    assert nan_start[3] == 20
    assert nan_start.size == 4

    # 3. check nan_end calculation
    assert nan_end[0] == 8
    assert nan_end[1] == 16
    assert nan_end[2] == 18
    assert nan_end[3] == 23
    assert nan_end.size == 4


    # 4a. check shortNan calculation (TH = 3)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 3)
    assert short_nan[0] == 15
    assert short_nan[1] == 16
    assert short_nan[2] == 18
    assert short_nan.size == 3

    # 4b. check short_nan calculation (TH = 10000)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 10000)
    assert short_nan.size == 12

    # 4c. check short_nan calculation (TH = 1)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 1)
    assert short_nan.size == 0

    # 5a. check long_nan calculation (TH = 4)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 4)
    assert long_nan[0] == 4
    assert long_nan[1] == 5
    assert long_nan[2] == 6
    assert long_nan[3] == 7
    assert long_nan[4] == 8
    assert long_nan[5] == 20
    assert long_nan[6] == 21
    assert long_nan[7] == 22
    assert long_nan[8] == 23
    assert long_nan.size == 9

    # 5b. check longNan calculation (TH = 10000)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 10000)
    assert long_nan.size == 0

    # 5c. check longNan calculation (TH = 1)
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 1)
    assert long_nan.size == 12

    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 0, 0)+timedelta(minutes=15), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.size,))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # 6. check what happens when no nans are present
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 1)
    assert short_nan.size == 0
    assert long_nan.size == 0
    assert nan_start.size == 0
    assert nan_end.size == 0

    d = {'t': t[0:2], 'glucose': [120, np.nan]}
    data = pd.DataFrame(data=d)

    # 7a. check what happens when 1 nan is present
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 1)
    assert short_nan.size == 0
    assert long_nan.size == 1
    assert nan_start.size == 1
    assert nan_end.size == 1

    # 7b. check what happens when 1 nan is present
    short_nan, long_nan, nan_start, nan_end = find_nan_islands(data, 2)
    assert short_nan.size == 1
    assert long_nan.size == 0
    assert nan_start.size == 1
    assert nan_end.size == 1


import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.utils import to_mmol_l


def test_to_mmol_l():
    """
    Unit test of to_mmol_l function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 15, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = np.nan
    glucose[1:3] = [1801.8, 3603.6]
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    #Tests
    results = to_mmol_l(data)
    assert type(results) is pd.DataFrame
    assert 't' in results.columns
    assert 'glucose' in results.columns
    assert np.isnan(results.glucose.values[0])
    assert np.round(results.glucose.values[1]*10)/10 == 100
    assert np.round(results.glucose.values[2]*10)/10 == 200



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.utils import glucose_vector_to_dataframe


def test_glucose_vector_to_dataframe():
    """
    Unit test of glucose_vector_to_dataframe function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t1 = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 15, 0), timedelta(minutes=5)).astype(
        datetime)
    t2 = np.arange(datetime(2000, 1, 1, 1, 0, 0), datetime(2000, 1, 1, 1, 15, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t1.shape[0],))
    glucose[0] = np.nan
    glucose[1:3] = [100, 200]

    #Tests
    results = glucose_vector_to_dataframe(glucose=glucose, sample_time=5)
    assert type(results) is pd.DataFrame
    assert 't' in results.columns
    assert 'glucose' in results.columns

    assert np.isnan(results.glucose.values[0])
    assert results.glucose.values[1] == 100
    assert results.glucose.values[2] == 200
    assert pd.to_datetime(results.t.values[0]).to_pydatetime() == t1[0]
    assert pd.to_datetime(results.t.values[1]).to_pydatetime() == t1[1]
    assert pd.to_datetime(results.t.values[2]).to_pydatetime() == t1[2]

    results = glucose_vector_to_dataframe(glucose=glucose, sample_time=5, start_time=t2[0])
    assert type(results) is pd.DataFrame
    assert 't' in results.columns
    assert 'glucose' in results.columns

    assert np.isnan(results.glucose.values[0])
    assert results.glucose.values[1] == 100
    assert results.glucose.values[2] == 200
    assert pd.to_datetime(results.t.values[0]).to_pydatetime() == t2[0]
    assert pd.to_datetime(results.t.values[1]).to_pydatetime() == t2[1]
    assert pd.to_datetime(results.t.values[2]).to_pydatetime() == t2[2]




import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.risk import dynamic_risk


def test_dynamic_risk():
    """
    Unit test of dynamic_risk function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 1, 0, 0), datetime(2000, 1, 1, 2, 5, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    glucose[11] = 260
    glucose[12] = 265

    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    #Tests

    # 1. check the first value of result (independent on any parameter)
    assert np.round(dynamic_risk(data)[0]*100)/100 == -36.42

    # 2. test the function with different amplification_function parameter
    results = dynamic_risk(data, amplification_function='exp')
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)/10)*10 == 5630
    assert np.round(results[-1]*100)/100 == 45.87

    # 3. test the function with different maximum_amplification parameter
    results = dynamic_risk(data, maximum_amplification=1.2)
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)*100)/100 == 3.07
    assert np.round(results[-1]*100)/100 == 28.44

    # 4. test the function with different amplification_rapidity parameter
    results = dynamic_risk(data, amplification_rapidity=2.5)
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)*100)/100 == 8.23
    assert np.round(results[-1]*100)/100 == 37.93

    # 5. test the function with different maximum_damping parameter
    results = dynamic_risk(data, maximum_damping=1.8)
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)*100)/100 == 12.71
    assert np.round(results[-1]*100)/100 == 118.7

    # 6. test the function with a combination of parameters
    results = dynamic_risk(data, amplification_function='tanh', maximum_amplification=2.2, amplification_rapidity=1.7, maximum_damping=1.1)
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)*100)/100 == 4.59
    assert np.round(results[-1]*100)/100 == 22.12

    # 7. test different order of parameters
    results = dynamic_risk(data, amplification_rapidity=1.7, maximum_amplification=2.2, maximum_damping=1.1,amplification_function='tanh')
    assert np.round(results[0]*100)/100 == -36.42
    assert np.round(np.nanmean(results)*100)/100 == 4.59
    assert np.round(results[-1]*100)/100 == 22.12

    # Set empty test data
    t = []
    glucose = []
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # 8. return nan if data is empty
    assert np.isnan(dynamic_risk(data))



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.py_agata import Agata


def test_agata_init():
    """
    Unit test of Agata constructor.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    #Tests
    agata = Agata(glycemic_target='diabetes')

    assert agata.glycemic_target == 'diabetes'



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.variability import glucose_roc


def test_glucose_roc():
    """
    Unit test of glucose_roc function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    result = glucose_roc(data)

    # 1. check if results is a pd.Dataframe
    assert type(result) is pd.DataFrame

    # 2. check the columns of results
    assert 't' in result.columns
    assert 'glucose_roc' in result.columns

    # 3: check that the first and last timestamps coincide with the one of the input
    assert result.t.values[0] == data.t.values[0]
    assert result.t.values[-1] == data.t.values[-1]

    # 4. check that the length coincides with the one of the input
    assert result.glucose_roc.values.size == data.glucose.values.size

    # 5. check results calculation
    assert np.isnan(result.glucose_roc.values[0])
    assert np.isnan(result.glucose_roc.values[1])
    assert np.isnan(result.glucose_roc.values[2])
    assert np.round(result.glucose_roc.values[3]*100)/100 == 2.67
    assert np.round(result.glucose_roc.values[4] * 100) / 100 == 4.67
    assert np.round(result.glucose_roc.values[5] * 100) / 100 == 4.67
    assert np.round(result.glucose_roc.values[6] * 100) / 100 == 8
    assert np.round(result.glucose_roc.values[7] * 100) / 100 == 5.33
    assert np.round(result.glucose_roc.values[8] * 100) / 100 == 9.33
    assert np.round(result.glucose_roc.values[9] * 100) / 100 == 4
    assert (np.isnan(result.glucose_roc.values[10]))


import numpy as np
import pandas as pd
from datetime import datetime, timedelta
from copy import copy

from py_agata.input_validator import *

def to_mmol_l(data):
    """
    Converts a pandas dataframe, numpy array or float containing the glucose data in mgl/dl to mmol/l

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mmol/l

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)

    return pd.DataFrame(data={'t': data.t.values, 'glucose': data.glucose.values/18.018})


def to_mg_dl(data):
    """
    Converts a pandas dataframe, numpy array or float containing the glucose data in mmol/l to mg/dl

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mmol/l

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)

    return pd.DataFrame(data={'t': data.t.values, 'glucose': data.glucose.values*18.018})


def glucose_time_vectors_to_dataframe(glucose, t):
    """
    Converts the two given vectors containing the glucose samples and the corresponding timestamps, respectively,
    in a dataframe.

    Parameters
    ----------
    glucose: np.ndarray
        A vector of double containing the glucose data (in mg/dl)
    t: np.ndarray
        A vector of datetime containing the timestamps

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_same_length_ndarray(glucose, t)
    check_ndarray_float_parameter(glucose)
    check_ndarray_datetime_parameter(t)

    return pd.DataFrame(data={'t': t, 'glucose': glucose})


def glucose_vector_to_dataframe(glucose, sample_time, start_time=datetime(2000, 1, 1, 0, 0, 0)):
    """
    Transform a vector containing glucose samples sampled on an homogeneous timegrid with, timestep
    `sample_time`, in a timetable. The resulting timetable timestamps will
    start from `start_time`. If start_time is not specified, 2000-01-01 00:00 is
    used as default.

    Parameters
    ----------
    glucose: np.ndarray
        A vector of double containing the glucose data (in mg/dl)
    sample_time: int
        The sample time of data (in min)
    start_time: datetime, optional, default: datetime(2000, 1, 1, 0, 0, 0)
        The first timestamp of the resulting timetable. If start_time is not provided, 2000-01-01 00:00, is used as default.

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_ndarray_float_parameter(glucose)
    check_int_parameter(sample_time)
    check_datetime_parameter(start_time)

    time_interval = timedelta(minutes=sample_time)
    end_time = start_time + timedelta(minutes=(glucose.size*sample_time - sample_time))
    t = pd.date_range(start_time, end_time, freq=time_interval)

    return pd.DataFrame(data={'t': t, 'glucose': glucose})


def read_dexcom_data(file, extension='xlsx'):
    """
    Reads data from a .xlsx or .csv file downloaded from the Dexcom CGM system and converts it in a timetable compatible
    with AGATA.

    Parameters
    ----------
    file: str
        A string containing the relative path to the file to be converted in a timetable compatible with AGATA.
    extension: str, optional, {'xlsx', 'csv'}, default: xlsx
        The extension of the raw file.

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_str_parameter(file)
    check_str_parameter(extension)

    df = pd.read_excel(file) if extension == 'xlsx' else pd.read_csv(file)

    egvs = np.where(df[df.columns[2]] == 'EGV')[0]
    g_raw = df[df.columns[7]][egvs]
    t_raw = df[df.columns[1]][egvs]


    t = np.empty(shape=(t_raw.size,), dtype=datetime)
    glucose = np.empty(shape=(g_raw.size,), dtype=float)

    count = 0
    for e in egvs:

        t[count] = datetime.strptime(t_raw[e], '%Y-%m-%dT%H:%M:%S')

        if type(g_raw[e]) is str:
            if g_raw[e] == 'Low':
                d = 39
            elif g_raw[e] == 'High':
                d = 401
        else:
            d = g_raw[e]
        glucose[count] = d
        count += 1

    t = t[0:count]
    glucose = glucose[0:count]

    data = pd.DataFrame(data={'t': t, 'glucose': glucose})
    data = data.sort_values(by='t')
    return data


def read_eversense_data(file, extension='xlsx'):
    """
    Reads data from a .xlsx or .csv file downloaded from the Eversense CGM system and converts it in a timetable compatible
    with AGATA.

    Parameters
    ----------
    file: str
        A string containing the relative path to the file to be converted in a timetable compatible with AGATA.
    extension: str, optional, {'xlsx', 'csv'}, default: xlsx
        The extension of the raw file.

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_str_parameter(file)
    check_str_parameter(extension)

    df = pd.read_excel(file) if extension == 'xlsx' else pd.read_csv(file)

    g_raw = df[df.columns[2]]
    t_date_raw = df[df.columns[0]]
    t_time_raw = df[df.columns[1]]
    unit_raw = df[df.columns[3]]

    t = np.empty(shape=(g_raw.size,), dtype=datetime)
    glucose = np.empty(shape=(g_raw.size,), dtype=float)

    count = 0
    for i in range(0, g_raw.size):

        glucose[count] = g_raw[i] if unit_raw[i] == 'mg/dL' else to_mg_dl(g_raw[i])

        t[count] = datetime.strptime(t_date_raw[i] + ' ' + t_time_raw[i], '%d-%B-%Y %I:%M %p')
        count += 1

    t = t[0:count]
    glucose = glucose[0:count]

    data = pd.DataFrame(data={'t': t, 'glucose': glucose})
    data = data.sort_values(by='t')
    return data


def read_freestyle_libre_data(file, extension='xlsx'):
    """
    Reads data from a .xlsx or .csv file downloaded from the Freestyle Libre CGM system and converts it in a timetable compatible
    with AGATA.

    Parameters
    ----------
    file: str
        A string containing the relative path to the file to be converted in a timetable compatible with AGATA.
    extension: str, optional, {'xlsx', 'csv'}, default: xlsx
        The extension of the raw file.

    Returns
    -------
    data: pd.DataFrame
        Pandas dataframe containing the glucose data in mg/dl

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Check input
    check_str_parameter(file)
    check_str_parameter(extension)

    df = pd.read_excel(file) if extension == 'xlsx' else pd.read_csv(file)

    g_raw = df[df.columns[6]][2:]
    t_date_raw = df[df.columns[3]][2:].to_dict()
    t_time_raw = df[df.columns[4]][2:].to_dict()

    t = np.empty(shape=(g_raw.size,), dtype=datetime)
    glucose = np.empty(shape=(g_raw.size,), dtype=float)

    count = 0
    for key in t_date_raw:

        glucose[count] = g_raw.values[count]
        t[count] = t_date_raw[key].to_pydatetime()+timedelta(minutes=t_time_raw[key].minute, hours=t_time_raw[key].hour)
        count += 1

    t = t[0:count]
    glucose = glucose[0:count]

    data = pd.DataFrame(data={'t': t, 'glucose': glucose})
    data = data.sort_values(by='t')
    return data


import numpy as np
from datetime import datetime
import pandas as pd


def check_dataframe(data):
    """
    Checks that data is a pd.Dataframe

    Parameters
    ----------
    data: pd.DataFrame
        Input data

    Returns
    -------
    is_ok: bool
        True if given data is ok

    Raises
    ------
    exception: exception
        If data is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(data) is pd.DataFrame:
        raise Exception("`data` is not a pd.Dataframe")

    return True


def check_data_columns(data):
    """
    Checks that data has a `glucose` and a `t` column

    Parameters
    ----------
    data: pd.DataFrame
        Input data

    Returns
    -------
    is_ok: bool
        True if given data is ok

    Raises
    ------
    exception: exception
        If data is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not 't' in data.columns.values:
        raise Exception("`data` must have a `t` column")
    if not 'glucose' in data.columns.values:
        raise Exception("`data` must have a `glucose` column")

    return True


def check_homogeneous_timegrid(data):
    """
    Checks that data has a homogeneous timegrid

    Parameters
    ----------
    data: pd.DataFrame
        Input data

    Returns
    -------
    is_ok: bool
        True if given data is ok

    Raises
    ------
    exception: exception
        If data is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    d = np.diff(data.t)
    if d.size == 0:
        return True

    if not np.all(d == d[0]):
        raise Exception("`data` has not an homogeneous timegrid")

    return True


def check_float_parameter(parameter):
    """
    Checks that parameter is a float

    Parameters
    ----------
    parameter: float
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is float:
        raise Exception("`parameter` is not a float")

    return True


def check_int_parameter(parameter):
    """
    Checks that parameter is a int

    Parameters
    ----------
    parameter: int
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is int:
        raise Exception("`parameter` is not an int")

    return True


def check_datetime_parameter(parameter):
    """
    Checks that parameter is a datetime

    Parameters
    ----------
    parameter: datetime
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is datetime:
        raise Exception("`parameter` is not a datetime")

    return True


def check_str_parameter(parameter):
    """
    Checks that parameter is a str

    Parameters
    ----------
    parameter: str
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is str:
        raise Exception("`parameter` is not a str")

    return True


def check_ndarray_float_parameter(parameter):
    """
    Checks that parameter is a np.ndarray of flaot

    Parameters
    ----------
    parameter: np.ndarray of float
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is np.ndarray:
        raise Exception("`parameter` is not a np.ndarray")

    if parameter.size == 0:
        return True

    if not type(parameter[0]) is np.float_:
        raise Exception("`parameter` must contain float values")

    return True


def check_ndarray_datetime_parameter(parameter):
    """
    Checks that parameter is a np.ndarray of datetime

    Parameters
    ----------
    parameter: np.ndarray of datetime
        Input parameter

    Returns
    -------
    is_ok: bool
        True if given parameter is ok

    Raises
    ------
    exception: exception
        If parameter is not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not type(parameter) is np.ndarray:
        raise Exception("`parameter` is not a np.ndarray")

    if parameter.size == 0:
        return True

    if not type(parameter[0]) is datetime:
        raise Exception("`parameter` must contain datetime values")

    return True


def check_same_length_ndarray(arr_1, arr_2):
    """
    Checks that arr_1 and arr_2 have the same length

    Parameters
    ----------
    arr_1: np.ndarray
        First array
    arr_2: np.ndarray
        Second array

    Returns
    -------
    is_ok: bool
        True if given arr_1 and arr_2 are ok

    Raises
    ------
    exception: exception
        If given arr_1 and arr_2 are not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not arr_1.size == arr_2.size:
        raise Exception("Given vectors must have the same length")

    return True


def check_same_length_dataframe(data_1, data_2):
    """
    Checks that data_1 and data_2 have the same length

    Parameters
    ----------
    data_1: pd.Dataframe
        First data frame
    data_2: pd.Dataframe
        Second data frame

    Returns
    -------
    is_ok: bool
        True if given data_1 and data_2 are ok

    Raises
    ------
    exception: exception
        If given data_1 and data_2 are not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not data_1.size == data_2.size:
        raise Exception("Given Dataframes must have the same length")

    return True


def check_comparable_data(data_1, data_2):
    """
    Checks that data_1 and data_2 are comparable

    Parameters
    ----------
    data_1: pd.Dataframe
        First data frame
    data_2: pd.Dataframe
        Second data frame

    Returns
    -------
    is_ok: bool
        True if given data_1 and data_2 are ok

    Raises
    ------
    exception: exception
        If given data_1 and data_2 are not valid

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    if not data_1.shape[0] == data_2.shape[0]:
        raise Exception("Given dataframes must have the same length")
    if data_1.shape[0] == 0:
        return True
    if not data_1.t.values[0] == data_2.t.values[0]:
        raise Exception("Given dataframes must start from the same timestamp")
    if not data_1.t.values[-1] == data_2.t.values[-1]:
        raise Exception("Given dataframes must end with the same timestamp")

    return True



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.inspection import find_hypoglycemic_events_by_level


def test_find_hypoglycemic_events_by_level():
    """
    Unit test of find_hypoglycemic_events_by_level function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 0, 0) + timedelta(minutes=425),
                  timedelta(minutes=5)).astype(datetime)
    glucose = np.ones(shape=(t.shape[0],))*120
    glucose[9:13] = np.ones(shape=(4,))*60
    glucose[29:60] = np.ones(shape=(31,))*60
    glucose[34:40] = np.ones(shape=(6,)) * 50
    glucose[44:50] = np.ones(shape=(6,)) * 50
    glucose[31:33] = [np.nan, np.nan]
    glucose[61:63] = np.ones(shape=(2,)) * 0
    glucose[69:72] = np.ones(shape=(3,)) * 60
    glucose[75:78] = np.ones(shape=(3,)) * 60
    glucose[79:82] = np.ones(shape=(3,)) * 60
    glucose[80] = np.nan

    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # Hypoglycemic events generated:
    #   - 2000-01-01 @ 00:45 - duration 00:20 (All + L1)
    #   - 2000-01-01 @ 02:45 - duration 02:30 (All)
    #   - 2000-01-01 @ 02:50 - duration 00:30 (L2)
    #   - 2000-01-01 @ 03:40 - duration 00:30 (L2)
    #   - 2000-01-01 @ 05:45 - duration 00:15 (All + L1)
    #   - 2000-01-01 @ 06:15 - duration 00:35 (All + L1)

    # 1: check results structure
    results = find_hypoglycemic_events_by_level(data)
    assert type(results) is dict
    assert type(results['hypo']) is dict
    assert type(results['l1']) is dict
    assert type(results['l2']) is dict
    assert results['hypo']['time_start'].size == results['hypo']['duration'].size
    assert results['hypo']['time_end'].size == results['hypo']['duration'].size
    assert results['l1']['time_start'].size == results['l1']['duration'].size
    assert results['l1']['time_end'].size == results['l1']['duration'].size
    assert results['l2']['time_start'].size == results['l2']['duration'].size
    assert results['l2']['time_end'].size == results['l2']['duration'].size

    # 2. check results (wth events)
    assert results['hypo']['time_start'][0] == datetime(2000, 1, 1, 0, 45, 0)
    assert results['hypo']['time_end'][0] == datetime(2000, 1, 1, 0, 45, 0) + timedelta(minutes=20)
    assert results['hypo']['duration'][0] == 20
    assert results['hypo']['time_start'][1] == datetime(2000, 1, 1, 2, 45, 0)
    assert results['hypo']['time_end'][1] == datetime(2000, 1, 1, 2, 45, 0) + timedelta(minutes=150)
    assert results['hypo']['duration'][1] == 150
    assert results['hypo']['time_start'][2] == datetime(2000, 1, 1, 5, 45, 0)
    assert results['hypo']['time_end'][2] == datetime(2000, 1, 1, 5, 45, 0) + timedelta(minutes=15)
    assert results['hypo']['duration'][2] == 15
    assert results['hypo']['time_start'][3] == datetime(2000, 1, 1, 6, 15, 0)
    assert results['hypo']['time_end'][3] == datetime(2000, 1, 1, 6, 15, 0) + timedelta(minutes=35)
    assert results['hypo']['duration'][3] == 35
    assert results['hypo']['time_start'].size == results['hypo']['duration'].size
    assert results['hypo']['time_end'].size == results['hypo']['duration'].size
    assert results['hypo']['time_start'].size == 4
    assert results['hypo']['mean_duration'] == 55
    assert results['hypo']['events_per_week'] == 96

    assert results['l1']['time_start'][0] == datetime(2000, 1, 1, 0, 45, 0)
    assert results['l1']['time_end'][0] == datetime(2000, 1, 1, 0, 45, 0) + timedelta(minutes=20)
    assert results['l1']['duration'][0] == 20
    assert results['l1']['time_start'][1] == datetime(2000, 1, 1, 5, 45, 0)
    assert results['l1']['time_end'][1] == datetime(2000, 1, 1, 5, 45, 0) + timedelta(minutes=15)
    assert results['l1']['duration'][1] == 15
    assert results['l1']['time_start'][2] == datetime(2000, 1, 1, 6, 15, 0)
    assert results['l1']['time_end'][2] == datetime(2000, 1, 1, 6, 15, 0) + timedelta(minutes=35)
    assert results['l1']['duration'][2] == 35
    assert results['l1']['time_start'].size == results['l1']['duration'].size
    assert results['l1']['time_end'].size == results['l1']['duration'].size
    assert results['l1']['time_start'].size == 3
    assert np.round(results['l1']['mean_duration']*100)/100 == 23.33
    assert results['l1']['events_per_week'] == 72

    assert results['l2']['time_start'][0] == datetime(2000, 1, 1, 2, 50, 0)
    assert results['l2']['time_end'][0] == datetime(2000, 1, 1, 2, 50, 0) + timedelta(minutes=30)
    assert results['l2']['duration'][0] == 30
    assert results['l2']['time_start'][1] == datetime(2000, 1, 1, 3, 40, 0)
    assert results['l2']['time_end'][1] == datetime(2000, 1, 1, 3, 40, 0) + timedelta(minutes=30)
    assert results['l2']['duration'][1] == 30
    assert results['l2']['time_start'].size == results['l2']['duration'].size
    assert results['l2']['time_end'].size == results['l2']['duration'].size
    assert results['l2']['time_start'].size == 2
    assert np.round(results['l2']['mean_duration'] * 100) / 100 == 30
    assert results['l2']['events_per_week'] == 48

    # 3. check results (without events)
    data.glucose = np.ones(shape=(data.glucose.values.size,))*120
    results = find_hypoglycemic_events_by_level(data)
    assert results['hypo']['time_start'].size == results['hypo']['duration'].size
    assert results['hypo']['time_end'].size == results['hypo']['duration'].size
    assert results['hypo']['time_start'].size == 0
    assert np.isnan(results['hypo']['mean_duration'])
    assert results['hypo']['events_per_week'] == 0

    assert results['l1']['time_start'].size == results['l1']['duration'].size
    assert results['l1']['time_end'].size == results['l1']['duration'].size
    assert results['l1']['time_start'].size == 0
    assert np.isnan(results['l1']['mean_duration'])
    assert results['l1']['events_per_week'] == 0

    assert results['l2']['time_start'].size == results['l2']['duration'].size
    assert results['l2']['time_end'].size == results['l2']['duration'].size
    assert results['l2']['time_start'].size == 0
    assert np.isnan(results['l2']['mean_duration'])
    assert results['l2']['events_per_week'] == 0

    # Set empty data
    d = {'t': [], 'glucose': []}
    data = pd.DataFrame(data=d)

    # 5. Test with empty data
    results = find_hypoglycemic_events_by_level(data)
    assert results['hypo']['time_start'].size == results['hypo']['duration'].size
    assert results['hypo']['time_end'].size == results['hypo']['duration'].size
    assert results['hypo']['time_start'].size == 0
    assert np.isnan(results['hypo']['mean_duration'])
    assert np.isnan(results['hypo']['events_per_week'])

    assert results['l1']['time_start'].size == results['l1']['duration'].size
    assert results['l1']['time_end'].size == results['l1']['duration'].size
    assert results['l1']['time_start'].size == 0
    assert np.isnan(results['l1']['mean_duration'])
    assert np.isnan(results['l1']['events_per_week'])

    assert results['l2']['time_start'].size == results['l2']['duration'].size
    assert results['l2']['time_end'].size == results['l2']['duration'].size
    assert results['l2']['time_start'].size == 0
    assert np.isnan(results['l2']['mean_duration'])
    assert np.isnan(results['l2']['events_per_week'])



import numpy as np
import pandas as pd
from datetime import datetime,timedelta

from py_agata.time_in_ranges import time_in_l1_hypoglycemia, time_in_l2_hypoglycemia, time_in_l1_hyperglycemia, time_in_l2_hyperglycemia
from py_agata.input_validator import *

def adrr(data):
    """
    Computes the average daily risk range (ADRR) of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl).

    Returns
    -------
    adrr: float
        the average daily risk range of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Kovatchev et al., "Evaluation of a new measure of blood glucose variability in
    diabetes", Diabetes Care, 2006, vol. 29, pp. 2433-2438. DOI: 10.2337/dc06-1085.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    if data.t.values.size == 0:
        return np.nan

    # Setup the formula parameters
    alpha = 1.084
    beta = 5.381
    gamma = 1.509
    th = 112.5

    # Get the first and last day limits
    first_day = pd.to_datetime(data.t.values[0]).to_pydatetime()
    first_day = first_day.replace(hour=0, minute=0, second=0)
    last_day = pd.to_datetime(data.t.values[-1]).to_pydatetime()
    last_day = last_day.replace(day=last_day.day+1,hour=0, minute=0, second=0)

    # Calculate the number of days and preallocate the daily max lbgi and hbgi
    n_days = (last_day-first_day).days
    max_lbgi_day = np.empty(shape=(n_days,))
    max_hbgi_day = np.empty(shape=(n_days,))

    for d in range(0,n_days):

        # Get the day of data
        low_limit = data.t >= (first_day + timedelta(days=d))
        high_limit = data.t < (first_day + timedelta(days=d+1))
        flags = np.logical_and(low_limit,high_limit)
        day_data = data.glucose.values[flags]

        # Get rid of nans
        non_nan_glucose = day_data[~np.isnan(day_data)]

        if not non_nan_glucose.size == 0:

            # Symmetrization
            f = gamma*(np.log(non_nan_glucose)**alpha-beta)

            # Risk computation
            rl = 10*(f**2)
            rl[non_nan_glucose > th] = 0
            rh = 10*(f**2)
            rh[non_nan_glucose < th] = 0

            # Get the max risks
            max_lbgi_day[d] = np.max(rl)
            max_hbgi_day[d] = np.max(rh)

        else:

            max_hbgi_day[d] = np.nan
            max_lbgi_day[d] = np.nan

    # Return adrr
    return np.nanmean(max_hbgi_day + max_lbgi_day)


def lbgi(data):
    """
    Computes the low blood glucose index (LBGI) of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl).

    Returns
    -------
    lbgi: float
        the low blood glucose index of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Kovatchev et al., "Evaluation of a new measure of blood glucose variability in
    diabetes", Diabetes Care, 2006, vol. 29, pp. 2433-2438. DOI: 10.2337/dc06-1085.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Setup the formula parameters
    alpha = 1.084
    beta = 5.381
    gamma = 1.509
    th = 112.5

    # Get rid of nans
    non_nan_glucose = data.glucose.values[~np.isnan(data.glucose.values)]

    # Symmetrization
    f = gamma*(np.log(non_nan_glucose)**alpha-beta)

    # Risk computation
    rl = 10*(f**2)
    rl[non_nan_glucose > th] = 0

    # Return lbgi
    return np.mean(rl)


def hbgi(data):
    """
    Computes the high blood glucose index (HBGI) of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl).

    Returns
    -------
    hbgi: float
        the high blood glucose index of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Kovatchev et al., "Evaluation of a new measure of blood glucose variability in
    diabetes", Diabetes Care, 2006, vol. 29, pp. 2433-2438. DOI: 10.2337/dc06-1085.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Setup the formula parameters
    alpha = 1.084
    beta = 5.381
    gamma = 1.509
    th = 112.5

    # Get rid of nans
    non_nan_glucose = data.glucose.values[~np.isnan(data.glucose.values)]

    # Symmetrization
    f = gamma*(np.log(non_nan_glucose)**alpha-beta)

    # Risk computation
    rh = 10*(f**2)
    rh[non_nan_glucose < th] = 0

    # Return hbgi
    return np.mean(rh)


def bgri(data):
    """
    Computes the blood glucose risk index (BGRI) of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl).

    Returns
    -------
    bgri: float
        the blood glucose risk index of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Kovatchev et al., "Evaluation of a new measure of blood glucose variability in
    diabetes", Diabetes Care, 2006, vol. 29, pp. 2433-2438. DOI: 10.2337/dc06-1085.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    # Return bgri
    return lbgi(data) + hbgi(data)


def gri(data):
    """
    Computes the blood glycemia risk index (GRI) of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl).

    Returns
    -------
    gri: float
        the glycemia risk index of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    Klonoff et al., "A Glycemia Risk Index (GRI) of hypoglycemia and
    hyperglycemia for continuous glucose monitoring validated by clinician
    ratings", Journal of Diabetes Science and Technology, 2022, pp. 1-17.
    DOI: 10.1177/19322968221085273.
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)

    #Compute metric
    v_low = time_in_l2_hypoglycemia(data) # VLow( < 54 mg / dL; < 3.0 mmol / L)
    low = time_in_l1_hypoglycemia(data) # Low(54–70 mg / dL; 3.0–3.9 mmol / L)
    v_high = time_in_l2_hyperglycemia(data) # VHigh( > 250 mg / dL; > 13.9 mmol / L)
    high = time_in_l1_hyperglycemia(data) # High( > 180–250 mg / dL; > 10.0–13.9 mmol / L)

    gri = (3.0 * v_low) + (2.4 * low) + (1.6 * v_high) + (0.8 * high)

    #Limit gri between 0 - 100 and return
    return np.min([gri, 100])


def dynamic_risk(data, amplification_function='tanh', maximum_amplification=2.5, amplification_rapidity=2., maximum_damping=0.6):
    """
    Computes the dynamic risk of the glucose concentration (ignoring nan values).

    Parameters
    ----------
    data: pd.DataFrame
        Pandas dataframe with a column `glucose` containing the glucose data
        to analyze (in mg/dl)
    amplification_function: str, {'exp', 'tanh'}, optional, default: 'tanh'
        Function to use to amplify the rate-of-change contribution to the dynamic risk
    maximum_amplification: float, optional, default: 2.5
        Intensity of amplification. Must be > 1
    amplification_rapidity: float, optional, default: 2
        Rapidity of the amplification. Must be > 0
    maximum_damping: float, optional, default: 0.6
        Damping of amplification. Must be > 0

    Returns
    -------
    dynamic_risk: float
        the dynamic risk of the glucose concentration.

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    S. Guerra et al., "A Dynamic Risk Measure from Continuous Glucose Monitoring Data", Diabetes
    Technology & Therapeutics, 2011, vol. 13, pp. 843-852. DOI: 10.1089/dia.2011.0006
    """
    # Check input
    check_dataframe(data)
    check_data_columns(data)
    check_homogeneous_timegrid(data)
    check_str_parameter(amplification_function)
    check_float_parameter(amplification_rapidity)
    check_float_parameter(maximum_damping)
    check_float_parameter(maximum_amplification)

    # If data is empty return null
    if data.shape[0] == 0:
        return np.nan

    # Initialization
    alpha = 1.084
    beta = 5.381
    gamma = 1.509
    dr_delta = (maximum_amplification - maximum_damping) / 2
    dr_beta = dr_delta + maximum_damping
    print((1 - dr_beta) / dr_delta)
    dr_gamma = np.arctanh(complex((1 - dr_beta) / dr_delta, 0))
    rl = np.zeros(shape=(data.glucose.values.size,))
    rh = np.zeros(shape=(data.glucose.values.size,))


    # Compute rate-of-change
    ts = (pd.to_datetime(data.t.values[1]).to_pydatetime() - pd.to_datetime(data.t.values[0]).to_pydatetime()).total_seconds()/60
    roc = np.diff(data.glucose.values)/ts
    roc = np.append(0,roc)

    # Symmetrization
    f = gamma * (np.log(data.glucose.values) ** alpha - beta)
    rl = 10 * (f ** 2)
    rl[f > 0] = 0
    rh = 10 * (f ** 2)
    rh[f < 0] = 0

    # Compute static risk
    sr = rh-rl
    modulation_factor = np.ones(shape=(data.glucose.values.size,))
    dr_over_dg = np.divide(10 * (gamma**2) * 2 * alpha * (np.log(data.glucose.values)**(2 * alpha - 1) - beta * np.log(data.glucose.values)**(alpha - 1)), data.glucose.values)

    # Compute dynamic risk and return it
    if amplification_function == 'tanh':
        modulation_factor = np.real(dr_delta*np.tanh(amplification_rapidity * np.multiply(dr_over_dg, roc) + dr_gamma)) + dr_beta
    elif amplification_function == 'exp':
        modulation_factor = np.exp(maximum_amplification * np.multiply(dr_over_dg, roc))
    return np.multiply(sr, modulation_factor)



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.error import rmse


def test_rmse():
    """
    Unit test of rmse function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 30
    glucose[1:3] = [70, 70]
    glucose[3] = 70
    glucose[4:6] = [130, 130]
    glucose[6:8] = [np.nan, np.nan]
    glucose[8:10] = [260, 260]
    glucose[10] = 260
    d = {'t': t, 'glucose': glucose}
    data_hat = pd.DataFrame(data=d)

    #Tests
    res = np.sqrt(np.mean(np.square(np.array([40, 50, 50, 80, 120, 120, 260, 260]) - np.array([30, 70, 70, 70, 130, 130, 260, 260]))))

    assert np.isnan(rmse(data,data_hat)) == False
    assert rmse(data,data_hat) == res

    d = {'t': [], 'glucose': []}
    data_hat = pd.DataFrame(data=d)
    d = {'t': [], 'glucose': []}
    data = pd.DataFrame(data=d)

    # Tests
    assert np.isnan(rmse(data, data_hat))

    d = {'t': [t[0]], 'glucose': [np.nan]}
    data_hat = pd.DataFrame(data=d)
    d = {'t': [t[0]], 'glucose': [120]}
    data = pd.DataFrame(data=d)

    # Tests
    assert np.isnan(rmse(data, data_hat))


import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.processing import retime_glucose


def test_retime_glucose():
    """
    Unit test of retime_glucose function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 0, 0)+timedelta(minutes=30), timedelta(minutes=5)).astype(
        datetime)
    t[0] = t[0].replace(second=17)
    t[1] = t[1].replace(second=33)
    t[2] = t[2].replace(second=58)
    t[3] = t[3].replace(second=58)
    t[4] = t[4].replace(second=10)
    t[5] = t[5].replace(second=59)

    glucose = np.ones(shape=(t.size,))
    glucose[0] = 40
    glucose[1] = 50
    glucose[2:4] = [np.nan, np.nan]
    glucose[4:6] = [120, 120]

    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    #Tests
    results = retime_glucose(data, 5)

    # 1. check retime (timestep = 5)

    assert results.glucose.values[0] == 40
    assert results.glucose.values[1] == 50
    assert np.isnan(results.glucose.values[2])
    assert np.isnan(results.glucose.values[3])
    assert results.glucose.values[4] == 120
    assert results.glucose.values[5] == 120

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[3]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[4]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[5]).to_pydatetime().second == 0

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().minute == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().minute == 5
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().minute == 10
    assert pd.to_datetime(results.t.values[3]).to_pydatetime().minute == 15
    assert pd.to_datetime(results.t.values[4]).to_pydatetime().minute == 20
    assert pd.to_datetime(results.t.values[5]).to_pydatetime().minute == 25

    # 2. check retime (timestep = 10)
    results = retime_glucose(data, 10)

    assert results.glucose.values[0] == 40
    assert results.glucose.values[1] == 50
    assert results.glucose.values[2] == 120

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().second == 0

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().minute == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().minute == 10
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().minute == 20

    # 3. check retime (timestep = 12)
    results = retime_glucose(data, 12)

    assert results.glucose.values[0] == 45
    assert np.isnan(results.glucose.values[1])
    assert results.glucose.values[2] == 120

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().second == 0
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().second == 0

    assert pd.to_datetime(results.t.values[0]).to_pydatetime().minute == 0
    assert pd.to_datetime(results.t.values[1]).to_pydatetime().minute == 12
    assert pd.to_datetime(results.t.values[2]).to_pydatetime().minute == 24



import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.input_validator import check_homogeneous_timegrid

import pytest

def test_check_homogeneous_timegrid():
    """
    Unit test of check_homogeneous_timegrid function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # Test not error
    assert check_homogeneous_timegrid(data)

    d = {'t': [], 'glucose': []}
    data = pd.DataFrame(data=d)

    # Test not error
    assert check_homogeneous_timegrid(data)


    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    t[-1] = datetime(2000, 1, 2, 0, 0, 0)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [50, 50]
    glucose[3] = 80
    glucose[4:6] = [120, 120]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # Test errors
    with pytest.raises(Exception):
        check_homogeneous_timegrid(data)


import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta

from py_agata.py_agata import Agata


def test_analyze_one_arm():
    """
    Unit test of Agata.analyze_one_arm function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """
    # Set test data
    t = np.arange(datetime(2000, 1, 1, 0, 0, 0), datetime(2000, 1, 1, 0, 55, 0), timedelta(minutes=5)).astype(
        datetime)
    glucose = np.zeros(shape=(t.shape[0],))
    glucose[0] = 40
    glucose[1:3] = [60, 60]
    glucose[3] = 80
    glucose[4:6] = [120, 150]
    glucose[6:8] = [200, 200]
    glucose[8:10] = [260, 260]
    glucose[10] = np.nan
    d = {'t': t, 'glucose': glucose}
    data = pd.DataFrame(data=d)

    # Tests

    # Diabetes
    agata = Agata(glycemic_target='diabetes')

    results = agata.analyze_one_arm(data=[data,data])

    # Variability
    metric_list_name = ['mean_glucose', 'median_glucose', 'std_glucose', 'cv_glucose', 'range_glucose',
                        'iqr_glucose',
                        'auc_glucose', 'gmi', 'cogi', 'conga', 'j_index', 'mage_index', 'mage_minus_index',
                        'mage_plus_index',
                        'ef_index', 'modd', 'sddm_index', 'sdw_index', 'std_glucose_roc', 'cvga']
    for m in metric_list_name:
        assert results["variability"][m]["values"].size == 2

    # Time in ranges
    metric_list_name = ['time_in_target', 'time_in_tight_target', 'time_in_hypoglycemia',
                            'time_in_l1_hypoglycemia', 'time_in_l2_hypoglycemia', 'time_in_hyperglycemia',
                            'time_in_l1_hyperglycemia', 'time_in_l2_hyperglycemia']
    for m in metric_list_name:
        assert results["time_in_ranges"][m]["values"].size == 2

    # Risk
    metric_list_name = ['adrr', 'lbgi', 'hbgi', 'bgri', 'gri']
    for m in metric_list_name:
        assert results["risk"][m]["values"].size == 2

    # Glycemic transformation
    metric_list_name = ['grade_score', 'grade_hypo_score', 'grade_hyper_score',
                            'grade_eu_score', 'igc', 'hypo_index','hyper_index',
                            'mr_index']
    for m in metric_list_name:
        assert results["glycemic_transformation"][m]["values"].size == 2

    # Data quality
    metric_list_name = ['number_days_of_observation', 'missing_glucose_percentage']
    for m in metric_list_name:
        assert results["data_quality"][m]["values"].size == 2

    # Events
    assert results["events"]["hyperglycemic_events"]["hyper"]["mean_duration"]["values"].size == 2
    assert results["events"]["hyperglycemic_events"]["hyper"]["events_per_week"]["values"].size == 2
    assert results["events"]["hyperglycemic_events"]["l1"]["mean_duration"]["values"].size == 2
    assert results["events"]["hyperglycemic_events"]["l1"]["events_per_week"]["values"].size == 2
    assert results["events"]["hyperglycemic_events"]["l2"]["mean_duration"]["values"].size == 2
    assert results["events"]["hyperglycemic_events"]["l2"]["events_per_week"]["values"].size == 2

    assert results["events"]["hypoglycemic_events"]["hypo"]["mean_duration"]["values"].size == 2
    assert results["events"]["hypoglycemic_events"]["hypo"]["events_per_week"]["values"].size == 2
    assert results["events"]["hypoglycemic_events"]["l1"]["mean_duration"]["values"].size == 2
    assert results["events"]["hypoglycemic_events"]["l1"]["events_per_week"]["values"].size == 2
    assert results["events"]["hypoglycemic_events"]["l2"]["mean_duration"]["values"].size == 2
    assert results["events"]["hypoglycemic_events"]["l2"]["events_per_week"]["values"].size == 2

    assert results["events"]["extended_hypoglycemic_events"]["mean_duration"]["values"].size == 2
    assert  results["events"]["extended_hypoglycemic_events"]["events_per_week"]["values"].size == 2


import pandas as pd
import numpy as np
import datetime
from datetime import datetime, timedelta
import os
from py_agata.utils import read_dexcom_data


def test_read_dexcom_data():
    """
    Unit test of read_dexcom_data function.

    Parameters
    ----------
    None

    Returns
    -------
    None

    Raises
    ------
    None

    See Also
    --------
    None

    Examples
    --------
    None

    References
    ----------
    None
    """

    file = os.path.join(os.path.abspath(''),'example','data','dexcom_example.xlsx')
    # d = {'t': time_range, 'glucose': data.glucose.values}
    # data = pd.DataFrame(data=d)

    #Tests
    data = read_dexcom_data(file)

    assert type(data) is pd.DataFrame
    assert 't' in data.columns
    assert 'glucose' in data.columns

    assert data.glucose.values.size == 3814
    assert np.all(np.isnan(data.glucose.values[8:10]))
    assert np.all(np.isnan(data.glucose.values[12:18]))
    assert data.glucose.values[10] == 401
    assert data.glucose.values[11] == 39
